#!/bin/bash
# Autobuilds wrapper script for MythTV package generation script
#  by Mario Limonciello
#  based off original weekly builds scripts by Mario Limonciello and Dave Walker
#
# This script will check out the MythTV Debian packaging script stored upstream.
# That script will generate source packages by checking out the appropriate bzr.
# The bzr will check out multiple git trees from upstream and assemble them.

setup_variables()
{
    #Common variables uses for all builds
    . ./variables_common.sh

    #Working directory
    export WORK_DIR="$(pwd)/work-mythtv"
}

clone_packaging()
{
    URL=git://github.com/MythTV/packaging.git
    if [ ! -d $WORK_DIR/packaging-scripts/.git ]; then
        echo "Cloning upstream MythTV build script from $URL"
        git clone $URL $WORK_DIR/packaging-scripts
    fi
}

upload()
{
    if [ "$DPUT" = "true" ]; then
        cd $WORK_DIR
        dput ppa:mythbuntu/${DPUT_PLACE} mythtv_${PKG_VERSION}_source.changes
    fi
}

build_dsc()
{
    SKIP=""
    BRANCH=$1
    PPA_HASH=""
    i=0
    #check out scripts
    cd $WORK_DIR/packaging-scripts
    git fetch
    git checkout $BRANCH
    git pull --rebase

    #echo "*********** These are blacklisted **********"
    #echo "$BLKLIST"
    #echo "********************************************"

    #Remove blacklisted builds
    #for item in $BLKLIST
    #do
    #    bldbranch=$(echo "$item" | cut -d';' -f1)
    #    release=$(echo "$item" | cut -d';' -f2)
    #
    #    if [ "$BRANCH" == "$bldbranch" ]; then
    #        DISTROS=`echo $DISTROS | sed "s/\b$release\b//g"`
    #    fi
    #done

    echo "*************** These distros **************"
    echo "$DISTROS"
    echo "********************************************"
    for dist in $DISTROS; do
        echo "*************** This distro now ************"
        echo "$dist"
        echo "********************************************"
        [ -z "$SKIP" ] || continue
        # increase counter for build number
        i=$(($i+1))
        export MYTHBUNTU_BUILD_NUMBER="$i"
        export UBUNTU_RELEASE="$dist"
        export DEBUILD_FLAGS="-S -i"
        export DEBIAN_SUFFIX="0ubuntu0mythbuntu$MYTHBUNTU_BUILD_NUMBER"
        if [ $MYTHBUNTU_BUILD_NUMBER -eq "1" ]; then
            # for the first upload, we include the orig.tar.gz
            # subsequent uploads don't need this because the orig.tar.gz
            # should have published by then
            export DEBUILD_FLAGS="$DEBUILD_FLAGS -sa"
        fi

        DATESTAMP=`date --utc '+%H:%M %b %e, %Y'`

        echo "Build number $MYTHBUNTU_BUILD_NUMBER, an $GIT_TYPE build for 0.$GIT_MAJOR_RELEASE.$GIT_MINOR_RELEASE"
        echo "This build is intended for Ubuntu $UBUNTU_RELEASE."
        echo "This build was started at $DATESTAMP UTC"

        cd $WORK_DIR/packaging-scripts
        deb/build-dsc.sh $BRANCH $WORK_DIR

        echo "dsc build status: $?"

        #figure out what version we just made
        cd $WORK_DIR/mythtv
        export PKG_VERSION=$(dpkg-parsechangelog | sed '/^Version/!d; s/.*[1-9]://')
        PKG_HASH=$(dpkg-parsechangelog | sed '/^Version/!d; s/.*~//; s/.*+//; s/-.*//;' | awk -F. '{print $3}')
        if [ -z "$PPA_HASH" ]; then
            if [ $GIT_MAJOR_RELEASE = "28" ]; then
                HASH_CHECK_PPA='0.28'
            elif [ $GIT_MAJOR_RELEASE = "29" ]; then
                HASH_CHECK_PPA='0.29'
            else
                HASH_CHECK_PPA=$GIT_MAJOR_RELEASE
            fi
            PPA_HASH=$(debian/PPA-published-git-checker.py $HASH_CHECK_PPA)
        fi
        if [ -n "$FORCED_REBUILD" ]; then
            if [ "$FORCED_REBUILD" = "$BRANCH" ]; then
                echo "Uploading forced rebuild, $FORCED_REBUILD"
                upload
            else
                echo "Not uploading to the PPA, $BRANCH is not marked in the force rebuild"
                SKIP=$PPA_HASH
            fi
        elif [ "$PPA_HASH" != "$PKG_HASH" ]; then
            #dput packages
            upload
        else
            echo "Not uploading to the PPA, this hash is already there ($PPA_HASH)"
            SKIP=$PPA_HASH
        fi
    done
}

cleanup()
{
    if [ -d $WORK_DIR/git/.git ] || [ -d $WORK_DIR/git/.pc ]; then
        if [ -n "$1" ]; then
            MSG="$1"
        else
            MSG="Cleaning up before build starts (in case)"
        fi
        echo "$MSG"
        cd $WORK_DIR/git
        quilt pop -a || true
        rm -rf .pc
        git checkout -- . || true
        for directory in mythplugins mythplugins/mythweb; do
            cd $WORK_DIR/git/$directory
            git checkout -- . || true
        done
    fi
}

trap_handler()
{
    LASTLINE="$1"            # argument 1: last line of error occurence
    LASTERR="$2"             # argument 2: error code of last command
    if [ -f /home/thomas/daily_mythtv.txt ]; then
        LOG=$(tail -n 200 /home/thomas/daily_mythtv.txt)
    fi
    mutt -s "MythTV auto build source failure ($(date))" thomas@mashos.com << EOF
    #cat << EOF
Script    :  ${0}
Line      :  ${LASTLINE}
Exit Code :  ${LASTERR}
Full log: http://people.ubuntu.com/~tgm4883/daily_mythtv.txt
tail -n 200:
${LOG}
EOF
    exit 1
}


##################
## Script Start ##
##################

#in case something goes wrong
set -E
trap 'trap_handler ${LINENO} $?' ERR

#setup variables
setup_variables

#if fresh start is true, cleanup so new clones will happen
if [ "$FRESH_START" = "true" ]; then
    rm -rf $WORK_DIR/*
else
    cleanup
fi

#check out upstream packaging script that we are wrapping, or update it
clone_packaging


######################
## Old Fixes Build  ##
######################
#init variables for old fixes
export GIT_MAJOR_RELEASE="$GIT_MAJOR_RELEASE_FIXES_OLD"
export GIT_MINOR_RELEASE="$GIT_MINOR_RELEASE_FIXES_OLD"
export DPUT_PLACE="fixes-old-building"
export DISTROS="$DISTROS_FIXES_OLD"
export GIT_TYPE="fixes"

#build and dput fixes builds
build_dsc "$GIT_TYPE/0.$GIT_MAJOR_RELEASE"

##################
## Fixes Build  ##
##################
#init variables for fixes
export GIT_MAJOR_RELEASE="$GIT_MAJOR_RELEASE_FIXES"
export GIT_MINOR_RELEASE="$GIT_MINOR_RELEASE_FIXES"
export DPUT_PLACE="fixes-building"
export DISTROS="$DISTROS_FIXES"
export GIT_TYPE="fixes"

#build and dput fixes builds
build_dsc "$GIT_TYPE/$GIT_MAJOR_RELEASE"

##################
## Master Build ##
##################
#init variables for master
export GIT_MAJOR_RELEASE="$GIT_MAJOR_RELEASE_MASTER"
export GIT_MINOR_RELEASE="$GIT_MINOR_RELEASE_MASTER"
export DPUT_PLACE="master-building"
export DISTROS="$DISTROS_MASTER"
export GIT_TYPE="master"

#build and dput master builds
build_dsc "master"

##################
##    Cleanup   ##
##################
if [ "$CLEANUP" = "true" ]; then
    cd $WORK_DIR
    rm -rf *.orig.tar.gz *.debian.tar.gz *.dsc *.build *.changes *source.upload
fi
